#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import time

import appier

from appier_extras import utils


class Prismic(object):
    """
    Partial (mixin) that adds the Prismic CMS proxy functionality
    to a controller or model that inherits from it.

    Most of the methods are meant to be used directly from the
    template or controller.
    """

    _prismic_cache = None
    """ The cache manager that is going to be used store
    prismic values, this should have a huge performance
    impact while accessing the values """

    @classmethod
    def prismic_clear(cls):
        cache = cls._get_prismic_cache()
        cache.clear()

    @classmethod
    def _prismic_deref(cls, entry):
        # verifies if the current value is not a dictionary
        # and that the sys type is not defined in the entry
        # value, if that's the case this is the termination
        # condition and then returns the current entry as the value
        if not isinstance(entry, dict):
            return entry
        if not "type" in entry:
            return entry
        if not "value" in entry:
            return entry

        # retrieves both the type and the (possible) multiple
        # values for the current entry, then retrieves the first
        # value as the "master" value for it
        type = entry["type"]
        values = entry["value"]

        if type == "Text":
            return values

        if type == "Boolean":
            return values

        if type == "Number":
            return values

        if type == "Select":
            return values

        if type == "Image":
            return values["main"]["url"]

        if type == "Link.web":
            return values["url"]

        if type == "StructuredText":
            return "\n".join([value["text"] for value in values])

        if type == "Group":
            group = []
            for value in values:
                value_d = dict(
                    [
                        (key, cls._prismic_deref(value))
                        for key, value in appier.legacy.iteritems(value)
                    ]
                )
                group.append(value_d)
            return group

        return entry

    @classmethod
    def _get_prismic_cache(cls, serialize=True, ref=None):
        ref = ref or Prismic
        if ref._prismic_cache:
            return ref._prismic_cache
        cache_engine = appier.conf("CACHE", "memory")
        cache_engine = appier.conf("CMS_CACHE_ENGINE", cache_engine)
        cache_engine = appier.conf("PRISMIC_CACHE_ENGINE", cache_engine)
        cache_engine = cache_engine.capitalize() + "Cache"
        cache_engine = getattr(appier, cache_engine)
        ref._prismic_cache = cache_engine.new(hash=True)
        if serialize:
            ref._prismic_cache = appier.SerializedCache(cls._prismic_cache)
        return ref._prismic_cache

    @classmethod
    def _filter_params(cls, kwargs):
        params = dict()
        for name in appier.legacy.keys(kwargs):
            if name in ("lang",):
                continue
            params[name] = kwargs.pop(name)
        return params

    def prismic_objects(self, *args, **kwargs):
        kwargs["multiple"] = True
        return self.prismic_value(*args, **kwargs)

    def prismic_value(
        self,
        key,
        default=None,
        cast=None,
        locale=None,
        timeout=86400,
        verify=False,
        multiple=False,
        *args,
        **kwargs
    ):
        # retrieves the reference to the parent class value to be used
        # to access class level methods
        cls = self.__class__

        # tries to retrieve the provided locale value and in case it
        # fails uses the locale associated with the current request
        # as a fallback (optimistic approach)
        locale = locale or self.request.locale

        # in case the cast value is set, tries to resolve it into the appropriate
        # cast operation using the default cast methods in the configuration module
        if cast:
            cast = appier.config._cast_r(cast)

        # in case e the locale value is provided it must be normalized
        # into the format expected by prismic API
        if locale:
            language, territory = locale.split("_", 1)
            locale = language + "-" + territory.upper()
            kwargs["lang"] = locale

        # prefixes the cache's key with locale when retrieving a value for
        # a specific locale, so that proper context is used for caching
        cache_key = locale + ":" + key if locale else key

        # in case there are named arguments provided, assumes that they must
        # be key value pairs for the filtering of the scope and appends the
        # extra filter string to the cache key
        if kwargs:
            cache_key += ":" + ",".join(
                "%s=%s" % (key, str(value))
                for key, value in appier.legacy.iteritems(kwargs)
            )

        # tries to retrieve the reference to the prismic cache engine
        # (singleton instance) and verifies if the key exists in it, returning
        # immediately the value if that's the case
        prismic_cache = cls._get_prismic_cache()
        if cache_key in prismic_cache:
            value = prismic_cache[cache_key]
            if cast and not value == None:
                value = cast(value)
            return value

        # determines if the provided key references a specific field/value on an
        # object or if it instead tries to retrieve the complete object specification
        # and then selects the appropriate retrieval method
        method = self._prismic_value if "." in key else self._prismic_object
        if multiple:
            method = self._prismic_objects

        # retrieve the value remotely and sets the value in the cache engine,
        # to avoid further retrievals later on
        value = method(key, default=default, verify=verify, *args, **kwargs)
        prismic_cache.set_item(cache_key, value, expires=time.time() + timeout)

        # runs the casting operation if required and then returns the final
        # value to the caller method
        if cast and not value == None:
            value = cast(value)
        return value

    def prismic_markdown(
        self,
        key,
        limit=10,
        default=None,
        verify=False,
        encoding="utf-8",
        options=dict(anchors=False, blank=False),
        *args,
        **kwargs
    ):
        value = self.prismic_value(
            key, limit=limit, default=default, verify=verify, *args, **kwargs
        )
        if not value:
            return value
        html_b = utils.MarkdownHTML.process_str(value, options=options)
        html_s = html_b.decode(encoding)
        return html_s

    @property
    def prismic_api(self):
        import prismic

        return prismic.API()

    def _prismic_object(self, *args, **kwargs):
        default = kwargs.get("default", None)
        kwargs["limit"] = 1
        objects = self._prismic_objects(*args, **kwargs)
        return objects[0] if objects else default

    def _prismic_objects(
        self, key, limit=10, default=[], verify=False, *args, **kwargs
    ):
        # retrieves the reference to the parent class value to be used
        # to access class level methods
        cls = self.__class__

        # runs a series of assertions for the provided key, raising exceptions
        # in case one of the pre-conditions is not met
        appier.verify(
            not "." in key,
            message="Malformed key '%s', must include both document type and key" % key,
            code=400,
        )

        # filters the keyword arguments based arguments retrieving only the valid
        # parameters to be used in the filters extension process
        params = cls._filter_params(kwargs)

        # sets the document type value as the provided key as we're trying
        # to retrieve an object instead of a field
        document_type = key

        # retrieves the complete set of items that meet the document type
        # criteria and selects the first one, default to an empty dictionary
        # in case no items exist for such content type
        try:
            query = ['[[at(document.type,"%s")]]' % document_type]
            query.extend(
                [
                    '[[at(my.%s.%s,"%s")]]' % (document_type, key, value)
                    for key, value in appier.legacy.iteritems(params)
                ]
            )
            entries = (
                self.prismic_api.search_documents(
                    q=query, page_size=limit, *args, **kwargs
                )
                or []
            )
        except Exception as exception:
            self.logger.warning("Problem while accessing prismic: %s" % exception)
            return default

        # creates the list that is going to hold the multiple entry
        # maps/objects that are going to store the dereferenced
        # prismic entries (after the filtering)
        entries_m = []

        # iterates over the complete set of entries, to be able to
        # creates the map/object with the dereferenced values
        for entry in entries:
            # creates the entry object/map that is going to be
            # populated with the complete set of dereferenced fields
            entry_m = dict()

            # retrieves the data part of the entry and the
            # field itself according to the document type
            data = entry.get("data", {})
            field = data.get(document_type, {})

            # iterates over the complete set of fields identifiers
            # and values and dereferences the values setting them
            # then in the current entry map/object
            for field_id, field_value in appier.legacy.iteritems(field):
                field_value = cls._prismic_deref(field_value)
                entry_m[field_id] = field_value

            # adds the entry map/object to the current list of entries,
            # this list should represent a simplified structure
            entries_m.append(entry_m)

        # returns the final processed list of entry objects with
        # the complete set of dereferenced values
        return entries_m

    def _prismic_value(self, key, limit=1, default=None, verify=False, *args, **kwargs):
        # retrieves the reference to the parent class value to be used
        # to access class level methods
        cls = self.__class__

        # runs a series of assertions for the provided key, raising exceptions
        # in case one of the pre-conditions is not met
        appier.verify(
            "." in key,
            message="Malformed key '%s', must include both document type and key" % key,
            code=400,
        )

        # filters the keyword arguments based arguments retrieving only the valid
        # parameters to be used in the filters extension process
        params = cls._filter_params(kwargs)

        # splits the provided key around the dot value (namespace oriented)
        # to obtain the document type and the field id
        document_type, field_id = key.split(".", 1)

        # retrieves the complete set of items that meet the document type
        # criteria and selects the first one, default to an empty dictionary
        # in case no items exist for such content type
        try:
            query = ['[[at(document.type,"%s")]]' % document_type]
            query.extend(
                [
                    '[[at(my.%s.%s,"%s")]]' % (document_type, key, value)
                    for key, value in appier.legacy.iteritems(params)
                ]
            )
            entries = (
                self.prismic_api.search_documents(
                    q=query, page_size=limit, *args, **kwargs
                )
                or []
            )
        except Exception as exception:
            self.logger.warning("Problem while accessing prismic: %s" % exception)
            return default

        # retrieves the complete set of items and in case there's at least
        # one returns the first one of it otherwise returns an empty dictionary
        entry = entries[0] if entries else dict()

        # retrieves the complete set of field for the entry and tries to retrieve
        # the requested field (by its identifier)
        data = entry.get("data", {})
        field = data.get(document_type, {})

        # verifies if the requested field exists raising an exception otherwise
        # this should ensure that the value exists in prismic
        if verify:
            appier.verify(
                field_id in field,
                message="'%s' not found" % field_id,
                exception=appier.NotFoundError,
            )

        # retrieves the value of the field requested with the provided identifier
        # defaulting to the provided default value in case it does not exists
        field_value = field.get(field_id, default)

        # runs the dereferencing process for the field value that is going to
        # be retrieved so the proper (expected) value may be returned to caller
        field_value = cls._prismic_deref(field_value)

        # returns the final dereferenced value to the caller method
        # this can be both a plain value or a sequence
        return field_value



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from appier_extras.parts.admin.models import base


class Locale(base.Base):
    locale = appier.field(index="all", default=True)

    context = appier.field(index="all")

    data_j = appier.field(
        type=dict, private=True, meta="longmap", description="JSON Data"
    )

    count_l = appier.field(type=int, initial=0, description="Count")

    @classmethod
    def setup(cls):
        super(Locale, cls).setup()
        cls._flush()
        appier.get_bus().bind("locale/reload", cls._flush)

    @classmethod
    def validate(cls):
        return super(Locale, cls).validate() + [
            appier.not_null("locale"),
            appier.not_empty("locale"),
            appier.is_regex("locale", "^[a-z]{2}(?:_[a-z]{2}(?:_[a-z]+)?)?$"),
            appier.not_null("data_j"),
            appier.not_empty("data_j"),
        ]

    @classmethod
    def list_names(cls):
        return ["locale", "context", "description", "count_l"]

    @classmethod
    def bundles_d(cls, locale=None):
        locales = cls.find_e(rules=False)
        return dict(
            [
                ((locale.locale, locale.context or None), locale.data_u)
                for locale in locales
            ]
        )

    @classmethod
    @appier.operation(
        name="Import Bundle",
        parameters=(
            ("JSON File", "file", "file"),
            ("Locale", "locale", str),
            ("Context", "context", str),
        ),
        factory=True,
    )
    def import_bundle_s(cls, file, locale, context=None, strict=False):
        context = context or None
        data_j = cls._json_read(file)
        conditions = cls._conditions(locale, context=context)
        locale_e = cls.get(rules=False, raise_e=False, **{"$and": conditions})
        if locale_e:
            locale_e.data_j.update(data_j)
        else:
            locale_e = cls(locale=locale, context=context, data_j=data_j)
        locale_e.save()
        return locale_e

    @classmethod
    @appier.operation(name="Import CSV", parameters=(("CSV File", "file", "file"),))
    def import_csv_s(cls, file):
        csv_reader = cls._csv_read(file)
        header = next(csv_reader)

        result = dict()
        locales_n = header[1:]
        for line in csv_reader:
            name, locales_v = line[0], line[1:]
            for locale_n, locale_v in zip(locales_n, locales_v):
                if not locale_v:
                    continue
                locale_m = result.get(locale_n, {})
                locale_m[name] = locale_v
                result[locale_n] = locale_m

        for locale, data_j in appier.legacy.iteritems(result):
            conditions = cls._conditions(locale)
            locale_e = cls.get(rules=False, raise_e=False, **{"$and": conditions})
            if locale_e:
                locale_e.data_j.update(data_j)
            else:
                locale_e = cls(locale=locale, data_j=data_j)
            locale_e.save()

    @classmethod
    @appier.link(name="Export CSV", context=True)
    def list_csv_url(cls, view=None, context=None, absolute=False):
        return appier.get_app().url_for(
            "admin.list_locales_csv", view=view, context=context, absolute=absolute
        )

    @classmethod
    @appier.view(name="Explorer", parameters=(("Context", "context", str),))
    def explorer_v(cls, context, *args, **kwargs):
        kwargs["sort"] = kwargs.get("sort", [("id", -1)])
        if context:
            kwargs.update(context=context)
        return appier.lazy_dict(
            model=cls,
            kwargs=kwargs,
            entities=appier.lazy(lambda: cls.find(*args, **kwargs)),
            page=appier.lazy(lambda: cls.paginate(*args, **kwargs)),
        )

    @classmethod
    def _flush(cls):
        # retrieves the dictionary of bundles currently
        # enabled in the data source
        bundles_d = cls.bundles_d()

        # iterates over the complete set of locale data pairs
        # in the bundles dictionary to set these bundles (locales)
        for (locale, context), data_j in appier.legacy.iteritems(bundles_d):
            appier.get_app()._register_bundle(data_j, locale, context=context)

    @classmethod
    def _escape(cls, data_j, target=".", sequence="::"):
        data_j = dict(data_j)
        for key, value in appier.legacy.items(data_j):
            _key = key.replace(target, sequence)
            if _key == key:
                continue
            data_j[_key] = value
            del data_j[key]
        return data_j

    @classmethod
    def _unescape(cls, data_j, target=".", sequence="::"):
        return cls._escape(data_j, target=sequence, sequence=target)

    @classmethod
    def _conditions(cls, locale, context=None):
        conditions = []
        conditions.append(dict(locale=locale))
        if context:
            conditions.append(dict(context=context))
        else:
            conditions.append({"$or": [dict(context=""), dict(context=None)]})
        return conditions

    def pre_create(self):
        base.Base.pre_create(self)
        self.count_l = len(self.data_j)
        self.data_j = self.__class__._escape(self.data_j)

    def pre_update(self):
        appier.Model.pre_update(self)
        self.count_l = len(self.data_j)
        self.data_j = self.__class__._escape(self.data_j)

    def post_create(self):
        base.Base.post_create(self)
        self.owner.trigger_bus("locale/reload")

    def post_update(self):
        base.Base.post_update(self)
        self.owner.trigger_bus("locale/reload")

    def post_delete(self):
        base.Base.post_delete(self)
        self.owner.trigger_bus("locale/reload")

    @appier.operation(name="Set Context", parameters=(("Context", "context", str),))
    def set_context_s(self, context):
        locale = self.reload(rules=False)
        locale.context = context
        locale.save()

    @appier.link(name="Export Bundle")
    def bundle_url(self, absolute=False):
        return appier.get_app().url_for(
            "admin.bundle_locale_json", id=self.id, absolute=absolute
        )

    @property
    def data_u(self):
        cls = self.__class__
        return cls._unescape(self.data_j)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from appier_extras import base

from . import util


class ReCaptchaPart(appier.Part):
    """
    Modular part class that provides the required infra-structure
    for the control of Google's reCAPTCHA service.

    Should be used with proper knowledge of the inner workings of
    the captcha mechanism to avoid any security problems.

    :see: https://developers.google.com/recaptcha
    """

    def version(self):
        return base.VERSION

    def load(self):
        appier.Part.load(self)

        self.owner.context["recaptcha"] = self.recaptcha
        self.owner.context["recaptcha_input"] = self.recaptcha_input
        self.owner.context["recaptcha_script"] = self.recaptcha_script
        self.owner.context["recaptcha_available"] = self.recaptcha_available

    def recaptcha(self, action="default", name="recaptcha_token", force=False):
        if not util._recaptcha_available() and not force:
            return ""
        return self.owner.escape_template(
            self._build_input(name) + self._build_script(action)
        )

    def recaptcha_input(self, name="recaptcha_token", force=False):
        if not util._recaptcha_available() and not force:
            return ""
        return self.owner.escape_template(self._build_input(name))

    def recaptcha_script(self, action="default", force=False):
        if not util._recaptcha_available() and not force:
            return ""
        return self.owner.escape_template(self._build_script(action))

    def recaptcha_available(self):
        return util._recaptcha_available()

    def _build_input(self, name):
        return '<input type="hidden" id="recaptcha-token" name="%s" />' % name

    def _build_script(self, action):
        recaptcha_key = appier.conf("RECAPTCHA_KEY", None)
        appier.verify(recaptcha_key, message="No reCAPTCHA site key provided")
        return (
            '<script type="application/javascript" src="https://www.google.com/recaptcha/api.js?render=%s"></script>'
            % recaptcha_key
            + '<script type="application/javascript">window.genRecaptcha = function () {'
            + "grecaptcha.ready(function () {"
            + 'var elements = document.querySelectorAll("[name=recaptcha_token], #recaptcha-token");'
            + "for (var index = 0; index < elements.length; index++) {"
            + "(function () {"
            + "var element = elements[index];"
            + 'grecaptcha.execute("%s", {action: "%s"}).then(function (token) {'
            % (recaptcha_key, action)
            + "element.value = token;"
            + "});"
            + "})();"
            + "}"
            + "});"
            + "};"
            + "window.genRecaptcha();"
            + "</script>"
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import base64
import unittest

import appier

import appier_extras


class ImageTest(unittest.TestCase):
    def test_resize_image(self):
        try:
            import PIL.Image
        except ImportError:
            self.skipTest("PIL not installed")

        image_data_b64 = b"iVBORw0KGgoAAAANSUhEUgAAAGQAAABkCAAAAABVicqIAAAAP0lEQVRo3u3NMQ0AAAgDMOZfNFPBQdIaaHbuRSKRSCQSiUQikUgkEolEIpFIJBKJRCKRSCQSiUQikUgkkv9JAZqQZAF1iHVBAAAAAElFTkSuQmCC"
        image_data = base64.b64decode(image_data_b64)

        image = PIL.Image.open(appier.legacy.BytesIO(image_data))
        self.assertEqual(image.size, (100, 100))
        self.assertEqual(image.format, "PNG")

        get_data, etag = appier_extras.resize_image(image_data, width=50, height=50)
        output_data = get_data()
        output_image = PIL.Image.open(appier.legacy.BytesIO(output_data))
        self.assertEqual(output_image.size, (50, 50))
        self.assertEqual(output_image.format, "PNG")
        self.assertEqual(etag, None)

        get_data, etag = appier_extras.resize_image(image_data, height=50)
        output_data = get_data()
        output_image = PIL.Image.open(appier.legacy.BytesIO(output_data))
        self.assertEqual(output_image.size, (50, 50))
        self.assertEqual(output_image.format, "PNG")
        self.assertEqual(etag, None)

        get_data, etag = appier_extras.resize_image(image_data, format="JPEG")
        output_data = get_data()
        output_image = PIL.Image.open(appier.legacy.BytesIO(output_data))
        self.assertEqual(output_image.size, (100, 100))
        self.assertEqual(output_image.format, "JPEG")
        self.assertEqual(etag, None)

        get_data, etag = appier_extras.resize_image(
            image_data, width=50, height=50, format="JPEG"
        )
        output_data = get_data()
        output_image = PIL.Image.open(appier.legacy.BytesIO(output_data))
        self.assertEqual(output_image.size, (50, 50))
        self.assertEqual(output_image.format, "JPEG")
        self.assertEqual(etag, None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier


def resize_image(
    data, etag=None, width=None, height=None, format=None, quality=None, resample=None
):
    def get_data():
        import PIL.Image

        input_stream = appier.legacy.BytesIO(data)
        output_stream = appier.legacy.BytesIO()

        image = PIL.Image.open(input_stream)

        try:
            target_width = width
            target_height = height
            target_format = format if format else image.format

            image_width, image_height = image.size
            if width:
                ratio = float(image_width) / float(width)
            elif height:
                ratio = float(image_height) / float(height)
            else:
                ratio = 1.0

            if not target_width:
                target_width = int(image_width * ratio)
            if not target_height:
                target_height = int(image_height * ratio)

            _resample = (
                (
                    PIL.Image.ANTIALIAS  # type: ignore
                    if hasattr(PIL.Image, "ANTIALIAS")
                    else (PIL.Image.LANCZOS if hasattr(PIL.Image, "LANCZOS") else None)  # type: ignore
                )
                if resample == None
                else resample
            )

            image.thumbnail((target_width, target_height), _resample)
            if quality:
                image.save(output_stream, target_format, quality=quality)
            else:
                image.save(output_stream, target_format)

            output_data = output_stream.getvalue()
        finally:
            input_stream.close()
            output_stream.close()
            image.close()

        return output_data

    etag = "%s-w%s-h%s" % (etag, str(width), str(height)) if etag else etag

    return (get_data, etag)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from . import base


class DiagRequest(base.DiagBase):
    address = appier.field(index="all", immutable=True)

    url = appier.field(index="all", immutable=True)

    method = appier.field(index="all", immutable=True)

    path = appier.field(index="all", immutable=True)

    query = appier.field(index="all", immutable=True)

    code = appier.field(type=int, index="all", immutable=True, description="Status")

    protocol = appier.field(index="all", immutable=True)

    content_type = appier.field(index="all", immutable=True)

    data = appier.field(immutable=True)

    duration = appier.field(type=float, index="all", immutable=True)

    in_length = appier.field(type=int, index="all", immutable=True)

    out_length = appier.field(type=int, index="all", immutable=True)

    browser = appier.field(index="all", immutable=True)

    headers = appier.field(type=dict, immutable=True)

    browser_info = appier.field(type=dict, immutable=True)

    meta_info = appier.field(type=dict, immutable=True)

    geo_info = appier.field(type=dict, immutable=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier

from appier_extras.parts.admin.models import base


class Search(base.Base):
    token = appier.field(index="all", default=True)

    target_id = appier.field(index="hashed", description="Target ID")

    target_cls = appier.field(index="hashed", description="Target Class")

    target_title = appier.field()

    target_description = appier.field()

    @classmethod
    def validate(cls):
        return super(Search, cls).validate() + [
            appier.not_null("token"),
            appier.not_empty("token"),
            appier.not_null("target_id"),
            appier.not_null("target_cls"),
            appier.not_empty("target_cls"),
            appier.not_null("target_title"),
            appier.not_empty("target_title"),
        ]

    @classmethod
    def list_names(cls):
        return ["id", "token", "target_id", "target_cls"]

    @classmethod
    def order_name(cls):
        return ["id", -1]

    @classmethod
    def is_indexed(cls):
        return False

    @classmethod
    def create_index(
        cls, token, target_id, target_cls, target_title, target_description=None
    ):
        token = cls._ensure_unicode(token)
        target_title = cls._ensure_unicode(target_title)
        target_description = cls._ensure_unicode(target_description)
        index = cls(
            token=token,
            target_id=target_id,
            target_cls=target_cls.__name__,
            target_title=target_title,
            target_description=target_description,
        )
        index.save()

    @classmethod
    def find_indexes(cls, target_id, target_cls, *args, **kwargs):
        target_cls = target_cls.__name__
        return cls.find(target_id=target_id, target_cls=target_cls, *args, **kwargs)

    @classmethod
    def delete_indexes(cls, target_id, target_cls, *args, **kwargs):
        indexes = cls.find_indexes(target_id, target_cls, build=False)
        for index in indexes:
            index.delete(*args, **kwargs)

    @classmethod
    def _plural(cls):
        return "Searches"

    @classmethod
    def _build(cls, model, map):
        super(Search, cls)._build(model, map)
        owner = appier.get_app()
        target_cls = model["target_cls"]
        target_id = model["target_id"]
        target = owner.get_model(target_cls)
        model["url"] = owner.url_for(
            "admin.show_entity", model=target._under(), _id=target_id
        )
        return model

    @classmethod
    def _ensure_unicode(self, value):
        if value == None:
            return value
        if appier.legacy.is_unicode(value):
            return value
        return appier.legacy.UNICODE(value)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import appier


class Facebook(object):
    def ensure_facebook(self):
        appier.ensure_pip("facebook", package="facebook_api")

    def has_facebook(self):
        try:
            import facebook
        except Exception:
            facebook = None
        if not facebook:
            return False
        if not appier.conf("FB_ID"):
            return False
        if not appier.conf("FB_SECRET"):
            return False
        return True

    def ensure_facebook_account(self, create=True, safe=True):
        api = self.get_facebook_api()
        user = api.self_user(fields=("id", "email"))
        email = user.get("email", None)
        facebook_id = user["id"]
        account = self.owner.admin_account.get(
            facebook_id=facebook_id, rules=False, raise_e=False
        )
        account = account or self.owner.admin_account.from_session()
        account = (
            account
            or email
            and self.owner.admin_account.get(email=email, rules=False, raise_e=False)
        )

        if safe and "fb.access_token" in self.session:
            del self.session["fb.access_token"]

        if not account:
            if not create:
                raise appier.NotFoundError(
                    message="No account found for Facebook account"
                )

            account = self.owner.admin_account(
                username=email,
                email=email,
                password=api.access_token,
                password_confirm=api.access_token,
                facebook_id=facebook_id,
                facebook_token=api.access_token,
                type=self.owner.admin_account.USER_TYPE,
            )
            account.save()
            account = account.reload(rules=False)

        if not account.facebook_id:
            account.facebook_id = facebook_id
            account.facebook_token = api.access_token
            account.save()

        if not account.facebook_token == api.access_token:
            account.facebook_token = api.access_token
            account.save()

        account.touch_login_s()
        account._set_account()

        return account

    def unset_facebook_account(self):
        account = self.owner.admin_account.from_session()
        account.facebook_id = None
        account.facebook_token = None
        account.save()

    def ensure_facebook_api(self, state=None, scope=None, refresh=False):
        access_token = self.session.get("fb.access_token", None)
        if access_token and not refresh:
            return
        api = self.get_facebook_api(scope=scope)
        return api.oauth_authorize(state=state)

    def get_facebook_api(self, scope=None):
        import facebook

        kwargs = dict()
        redirect_url = self.url_for("admin.oauth_facebook", absolute=True)
        access_token = self.session and self.session.get("fb.access_token", None)
        if scope:
            kwargs["scope"] = scope
        return facebook.API(
            client_id=appier.conf("FB_ID"),
            client_secret=appier.conf("FB_SECRET"),
            redirect_url=redirect_url,
            access_token=access_token,
            **kwargs
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2024 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__copyright__ = "Copyright (c) 2008-2024 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier
import appier_extras


class OAuthClientTest(unittest.TestCase):
    def setUp(self):
        self.app = appier.App(
            parts=(appier_extras.admin.AdminPart,), session_c=appier.MemorySession
        )

    def tearDown(self):
        self.app.unload()
        adapter = appier.get_adapter()
        adapter.drop_db()

    def test_basic(self):
        oauth_client = appier_extras.admin.OAuthClient()
        oauth_client.name = "name"
        oauth_client.redirect_uri = "http://localhost/oauth"
        oauth_client.save()

        self.assertNotEqual(oauth_client.id, None)
        self.assertNotEqual(oauth_client.client_id, None)
        self.assertNotEqual(oauth_client.client_secret, None)
        self.assertEqual(oauth_client.name, "name")
        self.assertEqual(oauth_client.redirect_uri, "http://localhost/oauth")
